 #
 #   $Id: UI.pm,v 1.23 2007/02/06 15:34:24 zarahg Exp $
 #
 #   This  file  is  part of the  PodWiki  web-based authoring tool.
 #
 #   By  accessing  this software,  PodWiki,  you are  duly informed
 #   of and  agree to be  bound  by the  conditions  described below
 #   in this notice:
 #
 #   This software product, PodWiki,  is developed by  Thomas Linden
 #   and   copyrighted  (C) 2003-2007   by  Thomas Linden,  with all
 #   rights reserved.
 #
 #   There is  no charge for  PodWiki software. You can redistribute
 #   it and/or modify it under  the terms of the  GNU General Public
 #   License, which is incorporated by reference herein.
 #
 #   PodWiki is distributed WITHOUT ANY WARRANTY,IMPLIED OR EXPRESS,
 #   OF  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that
 #   the  use of it will not infringe on any third party's intellec-
 #   tual property rights.
 #
 #   You  should  have  received a  copy  of  the GNU General Public
 #   License along with PodWiki.  Copies  can also be obtained from:
 #
 #     http://www.gnu.org/copyleft/gpl.html
 #
 #   or by writing to:
 #
 #     Free Software Foundation, Inc.
 #     59 Temple Place, Suite 330
 #     Boston, MA 02111-1307
 #     USA
 #
 #   Or contact:
 #
 #    "Thomas Linden" <tom@daemon.de>
 #


#
# this module provides functions which can be directly
# used by the user via P<state:WHAT> or L<state:WHAT>
#


package PodWiki::UI;

$PodWiki::UI::VERSION = "1.02";



use lib qw(. lib);

use strict;
no strict "refs";

use Exporter;
use Data::Dumper;
use CGI::Cookie;


use vars qw(@ISA @EXPORT @EXPORT_OK @EXPORT_TAGS);

@ISA=qw(Exporter);
@EXPORT=qw(jump info newest pageindex fileindex toc_by_attribute toc cloud tags_per_page);
@EXPORT_OK=qw();
@EXPORT_TAGS=();



sub jump {
  # print a select box to jump directly to a page
  opendir D, "$PodWiki::Runner::config->{data}" or
    die "Could not read DATA directory $PodWiki::Runner::config->{data}: $!\n";
  print qq(<form name="jump" title="jump" action="$PodWiki::Runner::me" method="get">
           <select name="page" onchange="document.forms['jump'].submit()">\n);
  foreach my $page (sort grep { $_ !~ /^\./ } readdir(D)) {
    next if($page =~ /\,v$/);
    next if($page =~ /\,lockuser$/);
    print qq(<option value="$page">$page</option>\n);
  }
  closedir D;
  print qq(</select><input type="submit" value="go"></form>\n);
}


sub info {
  my $rcs = new PodWiki::Revision(@PodWiki::Runner::rcs, "$PodWiki::Runner::page");
  my $revision = $rcs->head;
  my @revisions = $rcs->revisions;
  print qq(<h3>PageInfo about ) . PodWiki::Runner->pod_link($PodWiki::Runner::page) .
        qq( <i>(Revision: $revision)</i> ) .
        PodWiki::Runner->state_link("Edit this page", "edit", $PodWiki::Runner::page) . 
        qq( </h3>);

  my %comment = $rcs->comments;
  my $previous = 0;
  my %prev;
  foreach my $rev (reverse @revisions) {
    if ($previous) {
      $prev{$rev} = $previous;
    }
    $previous = $rev;
  }

  foreach my $rev (@revisions) {
    my $revdate = scalar localtime($rcs->revdate($rev));
    my $state   = $rcs->state($rev);
    my $author  = $rcs->author($rev);
    print qq(
     <table border="0" bgcolor="#000000" cellspacing="1" width=60%"><tr><td>
      <table border="0" cellspacing="1" cellpadding="5" width="100%">
      <tr>
        <th bgcolor="#cfcfcf" align="right" width="20%">Revision</th>
        <td bgcolor="#ffffff">
         <table border="0"><tr>
         <td>$rev</td>
         <td>
         <form title="activity" action="$PodWiki::Runner::me" method="get">
          <input type="hidden" name="page" value="$PodWiki::Runner::page">
          <input type="hidden" name="revision" value="$rev">
          <input type="hidden" name="state" value="checkout">
          <input type="submit" name="submit" value="Restore">
         </form>
         </td>
         <td>) .
         PodWiki::Runner->state_link("View", "view", $PodWiki::Runner::page, "revision=$rev");

    if (exists $prev{$rev}) {
      print PodWiki::Runner->state_link("diff to $prev{$rev}", "diff", $PodWiki::Runner::page, "previous=$prev{$rev}&current=$rev", "dotted");
    }

    print qq(
         </td></tr></table>
        </td>
      </tr>
      <tr>
        <th bgcolor="#cfcfcf" align="right">Date</th><td bgcolor="#ffffff">$revdate</td>
      </tr>
      <tr>
        <th bgcolor="#cfcfcf" align="right">State</th><td bgcolor="#ffffff">$state</td>
      </tr>
      <tr>
        <th bgcolor="#cfcfcf" align="right">Author</th><td bgcolor="#ffffff">$author</td>
      </tr>
      <tr>
        <th bgcolor="#cfcfcf" align="right">Comment</th><td bgcolor="#ffffff">$comment{$rev}</td>
      </tr>
      </table></td></tr></table><p>&nbsp;</p>
    );
    $previous = $rev;
  }
}

sub pageindex {
  # build an index of all existing pages
  my @include;
  my $title = "Page Index";
  if(exists $PodWiki::Runner::input->{template}) {
    &PodWiki::Runner::tie_templates;
    @include = split /\s*,\s*/, $PodWiki::Runner::templates{$PodWiki::Runner::input->{template}}->{files};
    $title = qq(Page Index for template "$PodWiki::Runner::input->{template}");
  }
  elsif ($PodWiki::Runner::input->{grep}) {
    $title = qq(Page Index for pages matching "$PodWiki::Runner::input->{grep}");
  }
  opendir D, "$PodWiki::Runner::config->{data}" or die "Could not read DATA directory $PodWiki::Runner::config->{data}: $!\n";
  print qq(
   <h3>$title</h3>
   <form name="grep" action="$PodWiki::Runner::me" method="POST">
     <input type="hidden" name="state" value="pageindex"/>
     <input type="text"   name="grep"  value=""/>
     <input type="submit" name="submit" value="list matching pages"/>
   </form>
   <table width="100%" border="0" bgcolor="#000000" cellspacing="1">
     <tr bgcolor="#cfcfcf">
      <th align="left">Page</th><th align="left">Revision</th><th align="left">Action</th>
     </tr>
  );

  foreach my $page (sort grep { $_ !~ /^\./ } readdir(D)) {
    next if($page =~ /\,v$/);
    next if($page =~ /\,lockuser$/);
    if(@include) {
      if(! grep { "$page" eq $_ } @include) { next; }
    }
    elsif ($PodWiki::Runner::input->{grep}) {
      next if($page !~ /$PodWiki::Runner::input->{grep}/i );
    }
    my $rcs = new PodWiki::Revision(@PodWiki::Runner::rcs, "$page");
    my $revision = $rcs->head;
    my $modified = scalar localtime($rcs->revdate);
    my $author   = $rcs->author;
    my $size     = (stat("$PodWiki::Runner::config->{data}/$page"))[7];
    my $link = PodWiki::Runner->pod_link("$page", "Author: $author, Modified: $modified, Size: $size");
    print qq(
      <tr bgcolor="#ffffff">
       <td>$link</td>
       <td>$revision</td>
       <td>) .
            PodWiki::Runner->state_link("source", "src", $page, "", "dotted") . "&nbsp;";
    if (! $PodWiki::Runner::config->{read_only}) {
      print
            PodWiki::Runner->state_link("edit", "edit", $page) . "&nbsp;" .
            PodWiki::Runner->state_link("remove", "remove", $page);
    }
    print qq(
       </td>
      </tr>
    );
  }
  print qq(</table><p>) .
    PodWiki::Runner->state_link("Maintain File Index (uploaded images/files)", "fileindex") .
    qq(</p>);
  closedir D;
}

sub fileindex {
  opendir D, "$PodWiki::Runner::config->{imgdir}" or die "Could not read IMAGES directory $PodWiki::Runner::config->{gfx}: $!\n";
    print qq(
   <p>&nbsp;</p>
   <h3>FileIndex</h3>
   <div>
  );
  
  my $thumbnail = "";
  foreach my $file (sort grep { $_ !~ /^\./ } readdir(D)) {
    next if($file =~ /thumbnail$/);
    my $modified = scalar localtime( (stat("$PodWiki::Runner::config->{imgdir}/$file"))[9] );
    if ($PodWiki::Runner::config->{image_magick}) {
      if ($file =~ /(jpg|jpeg|png|gif|bmp|xpm|tiff)$/i) {  
        $thumbnail = qq(<img src="$PodWiki::Runner::me?state=imagethumbnail&amp;image=$file" border="0" title="${file} modified $modified"/>);
      }
    }
    print qq(<div style="float: left; border: 1px solid #ccc; padding: 5px; width: 80px; height: 80px;"><a 
      href="$PodWiki::Runner::config->{gfx}/$file" target="__new__">${thumbnail}</a><br/>);
    if (! $PodWiki::Runner::config->{read_only}) {
      print PodWiki::Runner->state_link("remove", "remove", "", "file=$file");
    }
    print qq(</div>);
    $thumbnail = "";
  }
  print qq(</div>);
  
  closedir D;
} 

sub newest {
  opendir D, "$PodWiki::Runner::config->{data}" or die "Could not read DATA directory $PodWiki::Runner::config->{data}: $!\n";
  print "<h3>Newest Pages</h3>\n";
  print "<ul>\n";

  my %file;
  foreach my $page (sort grep { $_ !~ /^\./ } readdir(D)) {
    next if($page =~ /\,v$/);
    next if($page =~ /\,lockuser$/);
    my $modified = (stat("$PodWiki::Runner::config->{data}/$page"))[9];
    $file{$page} = $modified;
  }

  my $counter = $PodWiki::Runner::input->{count} || 5;
  foreach my $page (sort { $file{$b} <=> $file{$a} } keys %file) {
    if ($counter) {
      my $modified = scalar localtime($file{$page});
      my $link = PodWiki::Runner->pod_link($page);
      print qq(
       <li><b>$modified</b> $link</li>
      );
    }
    else {
      last;
    }
    $counter--;
  }

  print "</ul>\n";
  closedir D;
}



sub toc {
  # generate a table of content
  my $space;
  print "<p>\n";
  foreach my $target (@{$PodWiki::Runner::head{$PodWiki::Runner::page}}) {
    $space = "&nbsp;&nbsp;&nbsp;&nbsp;" x $target->{level};
    print qq( $space<a href="#$target->{target}">$target->{title}</a><br />\n);
  }
  print "</p><p>&nbsp;</p>\n";
}

sub cloud {
  # generate a tag/index cloud
  my %cloud;
  my ($max, $step);
  PodWiki::Runner::tie_index();
  foreach my $tag (sort keys %PodWiki::Runner::index) {
    foreach my $page (keys %{$PodWiki::Runner::index{$tag}}) {
      $cloud{$tag}->{num} ++;
      if ($cloud{$tag}->{num} > $max) {
        $max = $cloud{$tag}->{num};
      }
    }
  }


  my $size  = 10; # start size in pixel
  my $sizes = 8; # how many steps to increase font size
  my $fac   = 3; # increment factor
  if($max <= $sizes) {
    $step = 1;
  }
  else {
    $step  = int($max / $sizes);
  }
  my $currentstep = 0;


  # max: 110, step: 13
  foreach my $tag (sort { $cloud{$a}->{num} <=> $cloud{$b}->{num} } keys %cloud) {
    STEP:
    if ($cloud{$tag}->{num} < ($currentstep + $step) ) {
      $cloud{$tag}->{size} = $size;
    }
    elsif ($cloud{$tag}->{num} > ($currentstep + $step)) {
      $size += $fac; 
      $currentstep += $step;
      goto STEP;
    }
    else {
      $size += $fac; 
      $currentstep += $step;
      $cloud{$tag}->{size} = $size;
    }
  }

  print qq(<div id="tagcloud">); 
  foreach my $tag (sort keys %cloud) {
    print qq(&nbsp;<font style="font-size: $cloud{$tag}->{size}px"> ) .
     PodWiki::Runner->state_link($tag, "dosearch", "", "string=$tag&submit-index=true") .
     qq(</font>&nbsp);
  }
  print qq(</div>);
}

sub toc_by_attribute {
  # generate a table of content by selected attribute
  # FIXME: support for toc by multiple attributes!!!
  my ($attribute, $value);
  foreach my $attr (keys %{$PodWiki::Runner::attributes}) {
    next if $attr =~ /^\./;
    if (exists $PodWiki::Runner::input->{$attr}) {
      # this one has been set by CGI
      $attribute = $attr;
      $value     = $PodWiki::Runner::input->{$attr};
    }
  }
  print "<p>\n";
  if (exists $PodWiki::Runner::attributes->{".classified"}->{$attribute}->{$value}) {
    foreach my $page (sort keys %{$PodWiki::Runner::attributes->{".classified"}->{$attribute}->{$value}}) {
      if (-e "$PodWiki::Runner::config->{data}/$page") {
        print qq(&nbsp;&nbsp;&nbsp;&nbsp;) .
        PodWiki::Runner->state_link($page, "", $page, "attribute=$value") .
	qq( <br />);
      }
      else {
	my $ref = $PodWiki::Runner::attributes->{".classified"};
	delete $ref->{$attribute}->{$value}->{$page};
	$PodWiki::Runner::attributes->{".classified"} = $ref;
      }
    }
  }
  delete $PodWiki::Runner::input->{$attribute}; # clean up
  delete $PodWiki::Runner::input->{attribute};
  print "</p>\n";
}

sub tags_per_page {
  use Data::Dumper;
  PodWiki::Runner::tie_index();
  my @tags;
  foreach my $tag (sort keys %PodWiki::Runner::index) {
    if( exists $PodWiki::Runner::index{$tag}->{$PodWiki::Runner::page}) {
      push @tags, $tag;
    }
  }

  if(@tags) {
    my $help = PodWiki::Runner->pod_link("Tags|PodWikiMarkup#Tags");
    print qq(<br/><br/><strong>$help for this page:</strong> );
    foreach my $tag (@tags) {
      print qq(&nbsp; ) . 
      PodWiki::Runner->state_link($tag, "dosearch", "", "submit-index=true&string=$tag") . qq(\n);
    }
  }
}

1;
